@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
 """
-This module contains the implementation of QTableView.
+This module contains the tables.
 """
-from functools import partial
 import webbrowser
 import shutil
 import os
-from subprocess import Popen
+from functools import partial
 
 from PyQt5.QtGui import QIcon, QCursor
 from PyQt5.QtCore import pyqtSignal, QThread, QTimer
@@ -20,7 +19,7 @@ from PyQt5.QtWidgets import (
     QInputDialog
 )
 
-from get_data import get_active_dir_str
+import get_data
 from dialogs import ConsoleDialog, ProgBarDialog
 from manage_pip import PipManager
 from creator import (
@@ -111,21 +110,21 @@ class VenvTable(QTableView):
             lambda: self.upgrade_pip(event)
         )
 
-        install_modules_action = QAction(
+        install_packages_action = QAction(
             QIcon.fromTheme("list-add"),
-            "&Install additional modules",
+            "&Install additional packages",
             self,
-            statusTip="Install additional modules"
+            statusTip="Install additional packages"
         )
-        install_modules_action.triggered.connect(
-            lambda: self.add_modules(event)
+        install_packages_action.triggered.connect(
+            lambda: self.add_packages(event)
         )
 
         install_requires_action = QAction(
             QIcon.fromTheme("list-add"),
             "Install from &requirements",
             self,
-            statusTip="Install modules from requirements"
+            statusTip="Install packages from requirements"
         )
         install_requires_action.triggered.connect(
             lambda: self.install_requires(event)
@@ -161,14 +160,14 @@ class VenvTable(QTableView):
             lambda: self.save_requires(event)
         )
 
-        list_modules_action = QAction(
+        list_packages_action = QAction(
             self.info_icon,
-            "&List installed modules",
+            "&List installed packages",
             self,
-            statusTip="List installed modules"
+            statusTip="List installed packages"
         )
-        list_modules_action.triggered.connect(
-            lambda: self.list_modules(event, style=1)
+        list_packages_action.triggered.connect(
+            lambda: self.list_packages(event, style=1)
         )
 
         freeze_action = QAction(
@@ -220,7 +219,7 @@ class VenvTable(QTableView):
 
         # install sub menu
         context_menu.addMenu(install_sub_menu)
-        install_sub_menu.addAction(install_modules_action)
+        install_sub_menu.addAction(install_packages_action)
         install_sub_menu.addAction(install_requires_action)
 
         # editable sub menu
@@ -232,7 +231,7 @@ class VenvTable(QTableView):
 
         # details sub meun
         context_menu.addMenu(details_sub_menu)
-        details_sub_menu.addAction(list_modules_action)
+        details_sub_menu.addAction(list_packages_action)
         details_sub_menu.addAction(freeze_action)
         details_sub_menu.addAction(pipdeptree_action)
 
@@ -244,6 +243,31 @@ class VenvTable(QTableView):
             context_menu.popup(QCursor.pos())
 
 
+    def valid_version(self, venv_path):
+        """Test wether the Python version required is installed.
+        """
+        cfg_file = os.path.join(venv_path, "pyvenv.cfg")
+
+        is_installed = get_data.get_pyvenv_cfg(cfg_file, "installed")
+        version = get_data.get_pyvenv_cfg(cfg_file, "version")
+        py_path = get_data.get_pyvenv_cfg(cfg_file, "py_path")
+        msg_txt = (
+            f"This environment requires {version} \nfrom {py_path} which is \nnot installed.\n"
+        )
+
+        if is_installed == "no":
+            msg_box = QMessageBox(
+                QMessageBox.Critical,
+                "Error",
+                msg_txt,
+                QMessageBox.Ok,
+                self
+            )
+            msg_box.exec_()
+            return False
+        return True
+
+
     def venv_exists(self, path):
         """
         Test wether the directory of the selected environment actually exists.
@@ -254,7 +278,7 @@ class VenvTable(QTableView):
         msg_box = QMessageBox(
             QMessageBox.Critical,
             "Error",
-            "The selected environment could not be found.",
+            "Selected environment could not be found.",
             QMessageBox.Ok,
             self
         )
@@ -264,20 +288,19 @@ class VenvTable(QTableView):
 
 
     def has_pip(self, venv_dir, venv_name):
-        """
-        Test if `pip` is installed.
+        """Test if `pip` is installed.
         """
         venv_path = os.path.join(venv_dir, venv_name)
         pip_binary = os.path.join(venv_path, "bin", "pip")
         has_pip = os.path.exists(pip_binary)
 
-        if self.venv_exists(venv_path):
+        if self.venv_exists(venv_path) and self.valid_version(venv_path):
             if has_pip:
                 return True
             QMessageBox.information(
                 self,
                 "Info",
-                "Environment has been created without pip."
+                "This environment hasn't Pip installed."
             )
             return False
         return False
@@ -294,10 +317,9 @@ class VenvTable(QTableView):
 
 
     def upgrade_pip(self, event):
+        """Run `pip install --upgrade pip` command.
         """
-        Run `pip install --upgrade pip` command.
-        """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
 
         if self.has_pip(active_dir, venv):
@@ -316,19 +338,19 @@ class VenvTable(QTableView):
                 self.console.console_window.clear()
 
 
-    def add_modules(self, event):
+    def add_packages(self, event):
         """
-        Install additional modules into the selected environment.
+        Install additional packages into the selected environment.
         """
         pass
 
 
     def install_requires(self, event):
         """
-        Install modules from a requirements file into the
+        Install packages from a requirements file into the
         selected environment.
         """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
 
         if self.has_pip(active_dir, venv):
@@ -356,10 +378,9 @@ class VenvTable(QTableView):
 
 
     def install_local(self, event):
+        """Install from a local project.
         """
-        Install from a local project.
-        """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
 
         project_dir = QFileDialog.getExistingDirectory(
@@ -386,17 +407,15 @@ class VenvTable(QTableView):
 
 
     def install_vsc(self, event):
-        """
-        Install from a VSC repository.
+        """Install from a VSC repository.
         """
         self.clone_process()
 
 
     def clone_process(self):
+        """Clone the repository.
         """
-        Clone the repository.
-        """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
         venv_bin = os.path.join(active_dir, venv, "bin", "python")
 
@@ -438,7 +457,7 @@ class VenvTable(QTableView):
         """
         Write the requirements of the selected environment to file.
         """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
         venv_dir = os.path.join(active_dir, venv)
 
@@ -452,26 +471,25 @@ class VenvTable(QTableView):
 
             if save_path != "":
                 # write 'pip freeze' output to selected file
+                print(f"[PROCESS]: Saving '{save_path}'...")
                 self.manager = PipManager(active_dir, venv)
                 self.manager.run_pip(cmds[2], [">", save_path])
 
                 # show an info message
-                print(f"[PROCESS]: Saved requirements in {save_path}...")
                 message_txt = (f"Saved requirements in \n{save_path}")
                 QMessageBox.information(self, "Saved", message_txt)
 
 
-    def list_modules(self, event, style):
+    def list_packages(self, event, style):
+        """Open console dialog and list the installed packages.
         """
-        Open console dialog and list the installed modules.
-        """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
 
         if self.has_pip(active_dir, venv):
-            self.console.setWindowTitle(f"Modules installed in:  {venv}")
+            self.console.setWindowTitle(f"Packages installed in:  {venv}")
 
-            print("[PROCESS]: Listing modules...")
+            print("[PROCESS]: Listing packages...")
             self.manager = PipManager(active_dir, f"'{venv}'")
             self.manager.run_pip(cmds[style])
             self.manager.started.connect(self.console.exec_)
@@ -485,28 +503,27 @@ class VenvTable(QTableView):
 
 
     def freeze_output(self, event, style):
+        """Print `pip freeze` output to console window.
         """
-        Print `pip freeze` output to console window.
-        """
-        self.list_modules(event, style)
+        self.list_packages(event, style)
 
 
     def pipdeptree_output(self, event, style):
         """
         Test if `pipdeptree` is installed and ask user wether to
-        install it if it's not. Then call `self.list_modules()`
+        install it if it's not. Then call `self.list_packages()`
         """
         message_txt = (
             "This requires the pipdeptree package\nto be installed.\n\n"
             "Do you want to install it?\n"
         )
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
         pipdeptree_binary = os.path.join(active_dir, venv, "bin", "pipdeptree")
         has_pipdeptree = os.path.exists(pipdeptree_binary)
 
         if has_pipdeptree:
-            self.list_modules(event, style)
+            self.list_packages(event, style)
         else:
             if self.has_pip(active_dir, venv):
                 msg_box_confirm = QMessageBox.question(
@@ -526,19 +543,18 @@ class VenvTable(QTableView):
                     self.manager.started.connect(self.progress_bar.exec_)
                     self.manager.finished.connect(self.progress_bar.close)
                     self.manager.process_stop()
-                    self.list_modules(event, style)
+                    self.list_packages(event, style)
 
 
     def open_venv_dir(self, event):
+        """Open the selected venv directory.
         """
-        Open the selected venv directory.
-        """
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv = self.get_selected_item()
         venv_dir = os.path.join(active_dir, venv)
 
         if os.path.isdir(venv_dir):
-            Popen(["xdg-open", venv_dir])
+            os.system(f"xdg-open '{venv_dir}'")
 
 
     def delete_venv(self, event):
@@ -547,7 +563,7 @@ class VenvTable(QTableView):
         delete from the context menu in venv table.
         """
         venv = self.get_selected_item()
-        active_dir = get_active_dir_str()
+        active_dir = get_data.get_active_dir_str()
         venv_path = os.path.join(active_dir, venv)
 
         if self.venv_exists(venv_path):
@@ -565,9 +581,7 @@ class VenvTable(QTableView):
 
 
 class ResultsTable(QTableView):
-    """
-    The table that lists the [PyPI](https://pypi.org/pypi) results on
-    the wizard's `Install Modules` page.
+    """Contains the results from PyPI.
     """
     context_triggered = pyqtSignal()
 
@@ -589,7 +603,7 @@ class ResultsTable(QTableView):
             statusTip="Install module"
         )
         self.context_menu.addAction(install_action)
-        # connect to install_module() in InstallModules() in wizard
+        # connect to install_package() in InstallPackages() in wizard
         install_action.triggered.connect(
             lambda: self.context_triggered.emit()
         )
@@ -611,8 +625,7 @@ class ResultsTable(QTableView):
 
 
     def get_selected_item(self):
-        """
-        Get the venv name of the selected row.
+        """Get the venv name of the selected row.
         """
         listed_items = self.selectionModel().selectedRows()
         for index in listed_items:
